@@ -28,20 +28,51 @@
 #include <vmm_devdrv.h>
 #include <vmm_stdio.h>
 #include <libs/vtemu.h>
+#include <libs/bitops.h>
 #include <arch_board.h>
 
 #include <hpet.h>
 
+#ifdef CONFIG_PCI
+extern int __init pci_arch_init(void);
+extern int __init pci_subsys_init(void);
+#endif
+
 #if defined(CONFIG_VTEMU)
 struct vtemu *x86_vt;
 #endif
 
-int arch_board_reset(void)
+#define KBRD_INTFREG	0x64
+#define KBRD_BIT_KDATA	0 /* keyboard data is in buffer */
+#define KBRD_BIT_UDATA	1 /* user data is in buffer */
+#define KBRD_IO		0x60 /* IO Port */
+#define KBRD_RESET	0xfe /* RESET CPU command */
+
+static int generic_reset(void)
 {
+	volatile unsigned long good;
+
+	arch_cpu_irq_disable();
+
+	/* clear all keyboard buffer (user & keyboard) */
+	do {
+		good = vmm_inb(KBRD_INTFREG); /* empty user data */
+		if (good & (1 << KBRD_BIT_KDATA))
+			vmm_inb(KBRD_IO); /* empty keyboard data */
+
+	} while (good & (1 << KBRD_BIT_UDATA));
+
+	/* toggle the CPU reset pin */
+	vmm_outb(KBRD_RESET, KBRD_INTFREG);
+
+	while (1) {
+		arch_cpu_wait_for_irq();
+	}
+
 	return VMM_EFAIL;
 }
 
-int arch_board_shutdown(void)
+static int generic_shutdown(void)
 {
 	return VMM_EFAIL;
 }
@@ -53,7 +84,11 @@ int __init arch_board_early_init(void)
 	rv = hpet_init();
 	BUG_ON(rv != VMM_OK);
 
-        return VMM_OK;
+	/* Register reset & shutdown callbacks */
+	vmm_register_system_reset(generic_reset);
+	vmm_register_system_shutdown(generic_shutdown);
+
+	return VMM_OK;
 }
 
 int __init arch_clocksource_init(void)
@@ -73,7 +108,7 @@ int __init arch_board_final_init(void)
 	int rc;
 	struct vmm_devtree_node *node;
 #if defined(CONFIG_VTEMU)
-	struct vmm_fb_info *info;
+	struct fb_info *info;
 #endif
 
 	/* All VMM API's are available here */
@@ -87,6 +122,7 @@ int __init arch_board_final_init(void)
 
 	/* Do probing using device driver framework */
 	rc = vmm_devdrv_probe(node);
+	vmm_devtree_dref_node(node);
 	if (rc) {
 		return rc;
 	}
@@ -95,9 +131,9 @@ int __init arch_board_final_init(void)
 	 * and make it our stdio device.
 	 */
 #if defined(CONFIG_VTEMU)
-	info = vmm_fb_get(0);
+	info = fb_find("fb0");
 	if (info) {
-		x86_vt = vtemu_create(info->dev->node->name, info, NULL);
+		x86_vt = vtemu_create(info->dev.name, info, NULL);
 		if (x86_vt) {
 			vmm_stdio_change_device(&x86_vt->cdev);
 		}
