@@ -25,6 +25,7 @@
 
 #include <vmm_types.h>
 #include <arch_regs.h>
+#include <cpu_asm_macros.h>
 
 /* 8259A interrupt controller ports. */
 #define INT_CTL         0x20	/* I/O port for interrupt controller */
@@ -39,6 +40,7 @@
 /* Hardware interrupt numbers. */
 #define NR_GATES		256
 #define NR_IRQ_VECTORS		NR_GATES
+#define USER_DEFINED_IRQ_BASE	32
 
 /* Interrupt Descriptor Table */
 /* Segment Selector and Offset (SSO) */
@@ -86,6 +88,22 @@ struct idt64_ptr {
 	u64 idt_base;
 } __packed;
 
+#define REGULAR_INT_STACK	4
+#define STACKFAULT_STACK	1
+#define DOUBLEFAULT_STACK	2
+#define NMI_STACK		3
+#define DEBUG_STACK		0
+#define MCE_STACK		5
+#define EXCEPTION_STACK		6
+#define N_EXCEPTION_STACKS	7
+
+/*
+ * Change any of the values below will entail a
+ * respective change in the linker script.
+ */
+#define IRQ_STACK_SIZE		0x1000UL
+#define EXEC_STACK_SIZE		0x2000UL
+
 /*
  * These values are just flag bits not the actual value of
  * type to be written to register.
@@ -178,27 +196,27 @@ struct tss64_desc {
 } __packed;
 
 /* Interrupt handlers. */
-extern void _irq0(void);
-extern void _irq1(void);
-extern void _irq2(void);
-extern void _irq3(void);
-extern void _irq4(void);
-extern void _irq5(void);
-extern void _irq6(void);
-extern void _irq7(void);
-extern void _irq8(void);
-extern void _irq9(void);
-extern void _irq10(void);
-extern void _irq11(void);
-extern void _irq12(void);
-extern void _irq13(void);
-extern void _irq14(void);
-extern void _irq16(void);
-extern void _irq17(void);
-extern void _irq18(void);
-extern void _irq19(void);
-extern void _irq128(void);
-extern void _generic_handler(void);
+extern void _exception_div_error(void);
+extern void _exception_debug(void);
+extern void _exception_bp(void);
+extern void _exception_ovf(void);
+extern void _exception_bounds(void);
+extern void _exception_inval_opc(void);
+extern void _exception_no_dev(void);
+extern void _exception_double_fault(void);
+extern void _exception_coproc_overrun(void);
+extern void _exception_inval_tss(void);
+extern void _exception_missing_seg(void);
+extern void _exception_missing_stack(void);
+extern void _exception_gpf(void);
+extern void _exception_coproc_err(void);
+extern void _exception_align_check(void);
+extern void _exception_machine_check(void);
+extern void _exception_simd_err(void);
+extern void _exception_nmi(void);
+extern void _exception_page_fault(void);
 extern void __IRQ_32(void);
+extern void _generic_handler(void);
 
+extern void reload_host_tss(void);
 #endif /* _CPU_INTERRYPTS_H__ */
