@@ -23,8 +23,13 @@
 
 #include <vmm_error.h>
 #include <vmm_stdio.h>
+#include <vmm_delay.h>
 #include <vmm_devtree.h>
 #include <vmm_manager.h>
+#include <vmm_scheduler.h>
+#include <vmm_host_ram.h>
+#include <vmm_host_vapool.h>
+#include <vmm_host_aspace.h>
 #include <vmm_modules.h>
 #include <vmm_cmdmgr.h>
 #include <arch_vcpu.h>
@@ -43,6 +48,9 @@ static void cmd_vcpu_usage(struct vmm_chardev *cdev)
 	vmm_cprintf(cdev, "Usage:\n");
 	vmm_cprintf(cdev, "   vcpu help\n");
 	vmm_cprintf(cdev, "   vcpu list\n");
+	vmm_cprintf(cdev, "   vcpu orphan_list\n");
+	vmm_cprintf(cdev, "   vcpu normal_list\n");
+	vmm_cprintf(cdev, "   vcpu monitor [<output_chardev_name>]\n");
 	vmm_cprintf(cdev, "   vcpu reset   <vcpu_id>\n");
 	vmm_cprintf(cdev, "   vcpu kick    <vcpu_id>\n");
 	vmm_cprintf(cdev, "   vcpu pause   <vcpu_id>\n");
@@ -52,20 +60,31 @@ static void cmd_vcpu_usage(struct vmm_chardev *cdev)
 	vmm_cprintf(cdev, "   vcpu dumpstat <vcpu_id>\n");
 }
 
-static int cmd_vcpu_help(struct vmm_chardev *cdev, int dummy)
+static int cmd_vcpu_help(struct vmm_chardev *cdev,
+			 int argc, char **argv)
 {
 	cmd_vcpu_usage(cdev);
 	return VMM_OK;
 }
 
-static int cmd_vcpu_list_iter(struct vmm_vcpu *vcpu, void *priv)
+struct vcpu_list_priv {
+	struct vmm_chardev *cdev;
+	bool normal;
+	bool orphan;
+};
+
+static int vcpu_list_iter(struct vmm_vcpu *vcpu, void *priv)
 {
+	u32 hcpu, afflen;
 	char state[10];
-	char path[256];
-#ifdef CONFIG_SMP
-	u32 hcpu;
-#endif
-	struct vmm_chardev *cdev = priv;
+	const struct vmm_cpumask *aff;
+	struct vcpu_list_priv *p = priv;
+	struct vmm_chardev *cdev = p->cdev;
+
+	if (!(vcpu->is_normal && p->normal) &&
+	    !(!vcpu->is_normal && p->orphan)) {
+		return VMM_OK;
+	}
 
 	switch (vmm_manager_vcpu_get_state(vcpu)) {
 	case VMM_VCPU_STATE_UNKNOWN:
@@ -96,19 +115,29 @@ static int cmd_vcpu_list_iter(struct vmm_vcpu *vcpu, void *priv)
 	vmm_cprintf(cdev, " %-6d", hcpu);
 #endif
 	vmm_cprintf(cdev, " %-7d %-10s %-17s", vcpu->priority, state, vcpu->name);
-	if (vcpu->node) {
-		vmm_devtree_getpath(path, vcpu->node);
-		vmm_cprintf(cdev, " %-34s\n", path); 
-	} else {
-		vmm_cprintf(cdev, " %-34s\n", "(NA)"); 
+	vmm_cprintf(cdev, " %s", "{");
+	aff = vmm_manager_vcpu_get_affinity(vcpu);
+	afflen = 0;
+	for_each_cpu(hcpu, aff) {
+		if (afflen) {
+			vmm_cprintf(cdev, ",");
+		}
+		vmm_cprintf(cdev, "%d", hcpu);
+		afflen++;
 	}
+	vmm_cprintf(cdev, "%s\n", "}");
 
 	return VMM_OK;
 }
 
-static int cmd_vcpu_list(struct vmm_chardev *cdev, int dummy)
+static int vcpu_list(struct vmm_chardev *cdev, bool normal, bool orphan)
 {
 	int rc;
+	struct vcpu_list_priv p;
+
+	p.cdev = cdev;
+	p.normal = normal;
+	p.orphan = orphan;
 
 	vmm_cprintf(cdev, "----------------------------------------"
 			  "---------------------------------------\n");
@@ -117,11 +146,11 @@ static int cmd_vcpu_list(struct vmm_chardev *cdev, int dummy)
 	vmm_cprintf(cdev, " %-6s", "CPU ");
 #endif
 	vmm_cprintf(cdev, " %-7s %-10s %-17s %-34s\n", 
-		    "Prio", "State", "Name", "Device Path");
+		    "Prio", "State", "Name", "Affinity");
 	vmm_cprintf(cdev, "----------------------------------------"
 			  "---------------------------------------\n");
 
-	rc = vmm_manager_vcpu_iterate(cmd_vcpu_list_iter, cdev);
+	rc = vmm_manager_vcpu_iterate(vcpu_list_iter, &p);
 
 	vmm_cprintf(cdev, "----------------------------------------"
 			  "---------------------------------------\n");
@@ -129,11 +158,138 @@ static int cmd_vcpu_list(struct vmm_chardev *cdev, int dummy)
 	return rc;
 }
 
-static int cmd_vcpu_reset(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_list(struct vmm_chardev *cdev,
+			 int argc, char **argv)
 {
-	int ret;
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	return vcpu_list(cdev, TRUE, TRUE);
+}
 
+static int cmd_vcpu_orphan_list(struct vmm_chardev *cdev,
+				int argc, char **argv)
+{
+	return vcpu_list(cdev, FALSE, TRUE);
+}
+
+static int cmd_vcpu_normal_list(struct vmm_chardev *cdev,
+				int argc, char **argv)
+{
+	return vcpu_list(cdev, TRUE, FALSE);
+}
+
+static int cmd_vcpu_monitor(struct vmm_chardev *cdev,
+			    int argc, char **argv)
+{
+	char ch;
+	bool skip_sleep;
+	bool found_escape_ch;
+	u32 i, c, util;
+	virtual_size_t vfree, vtotal;
+	physical_size_t pfree, ptotal;
+	struct vmm_chardev *ocdev = NULL;
+
+	if (argc) {
+		ocdev = vmm_chardev_find(argv[0]);
+	}
+
+	if (!ocdev) {
+		ocdev = cdev;
+	}
+
+	while (1) {
+		/* Reset cursor positon using VT100 command */
+		vmm_cputs(ocdev, "\e[H");
+
+		/* Clear entire screen using VT100 command */
+		vmm_cputs(ocdev, "\e[J");
+
+		/* Print CPU usage */
+		i = 0;
+		for_each_online_cpu(c) {
+			vmm_cprintf(ocdev, "CPU%d:", c);
+			util = udiv64(vmm_scheduler_idle_time(c) * 1000,
+				      vmm_scheduler_get_sample_period(c));
+			util = (util > 1000) ? 1000 : util;
+			util = 1000 - util;
+			vmm_cprintf(ocdev, " %d.%01d%%  ",
+				    udiv32(util, 10), umod32(util, 10));
+			i++;
+			if (i % 4 == 0) {
+				vmm_cputs(ocdev, "\n");
+			}
+		}
+		if (i % 4) {
+			vmm_cputs(ocdev, "\n");
+		}
+
+		/* Print VAPOOL usage */
+		vfree = vmm_host_vapool_free_page_count();
+		vfree *= VMM_PAGE_SIZE;
+		vtotal = vmm_host_vapool_total_page_count();
+		vtotal *= VMM_PAGE_SIZE;
+		if (sizeof(virtual_size_t) == sizeof(u64)) {
+			vmm_cprintf(ocdev, "VAPOOL: free %llKiB  "
+				"used %llKiB  total %llKiB\n",
+				vfree/1024, (vtotal-vfree)/1024, vtotal/1024);
+		} else {
+			vmm_cprintf(ocdev, "VAPOOL: free %dKiB  "
+				"used %dKiB  total %dKiB\n",
+				vfree/1024, (vtotal-vfree)/1024, vtotal/1024);
+		}
+		/* Print RAM usage */
+		pfree = vmm_host_ram_free_frame_count();
+		pfree *= VMM_PAGE_SIZE;
+		ptotal = vmm_host_ram_total_frame_count();
+		ptotal *= VMM_PAGE_SIZE;
+		if (sizeof(physical_size_t) == sizeof(u64)) {
+			vmm_cprintf(ocdev, "RAM: free %llKiB  "
+				"used %llKiB  total %llKiB\n",
+				pfree/1024, (ptotal-pfree)/1024, ptotal/1024);
+		} else {
+			vmm_cprintf(ocdev, "RAM: free %dKiB  "
+				"used %dKiB  total %dKiB\n",
+				pfree/1024, (ptotal-pfree)/1024, ptotal/1024);
+		}
+
+		/* Print VCPU list */
+		vcpu_list(ocdev, TRUE, TRUE);
+
+		/* Look for escape character 'q' */
+		ch = 0;
+		skip_sleep = FALSE;
+		found_escape_ch = FALSE;
+		while (!vmm_scanchars(cdev, &ch, 1, FALSE)) {
+			skip_sleep = TRUE;
+			if (ch == 'q') {
+				found_escape_ch = TRUE;
+				break;
+			}
+		}
+		if (found_escape_ch) {
+			break;
+		}
+
+		/* Sleep for 1 seconds */
+		if (!skip_sleep) {
+			vmm_ssleep(1);
+		}
+	}
+
+	return VMM_OK;
+}
+
+static int cmd_vcpu_reset(struct vmm_chardev *cdev,
+			  int argc, char **argv)
+{
+	int ret, id;
+	struct vmm_vcpu *vcpu;
+
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -148,11 +304,19 @@ static int cmd_vcpu_reset(struct vmm_chardev *cdev, int id)
 	return ret;
 }
 
-static int cmd_vcpu_kick(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_kick(struct vmm_chardev *cdev,
+			 int argc, char **argv)
 {
-	int ret;
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	int ret, id;
+	struct vmm_vcpu *vcpu;
 
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -167,11 +331,19 @@ static int cmd_vcpu_kick(struct vmm_chardev *cdev, int id)
 	return ret;
 }
 
-static int cmd_vcpu_pause(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_pause(struct vmm_chardev *cdev,
+			  int argc, char **argv)
 {
-	int ret;
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	int ret, id;
+	struct vmm_vcpu *vcpu;
 
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -186,11 +358,19 @@ static int cmd_vcpu_pause(struct vmm_chardev *cdev, int id)
 	return ret;
 }
 
-static int cmd_vcpu_resume(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_resume(struct vmm_chardev *cdev,
+			   int argc, char **argv)
 {
-	int ret;
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	int ret, id;
+	struct vmm_vcpu *vcpu;
 
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -206,11 +386,19 @@ static int cmd_vcpu_resume(struct vmm_chardev *cdev, int id)
 	return ret;
 }
 
-static int cmd_vcpu_halt(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_halt(struct vmm_chardev *cdev,
+			 int argc, char **argv)
 {
-	int ret;
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	int ret, id;
+	struct vmm_vcpu *vcpu;
 
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -225,10 +413,19 @@ static int cmd_vcpu_halt(struct vmm_chardev *cdev, int id)
 	return ret;
 }
 
-static int cmd_vcpu_dumpreg(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_dumpreg(struct vmm_chardev *cdev,
+			    int argc, char **argv)
 {
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	int id;
+	struct vmm_vcpu *vcpu;
 
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -258,16 +455,24 @@ static void nsecs_to_hhmmsstt(u64 nsecs,
 	nsecs = udiv64(nsecs, 60ULL);
 }
 
-static int cmd_vcpu_dumpstat(struct vmm_chardev *cdev, int id)
+static int cmd_vcpu_dumpstat(struct vmm_chardev *cdev,
+			     int argc, char **argv)
 {
-	int ret;
+	int ret, id;
 	u8 priority;
 	u32 h, m, s, ms;
 	u32 state, hcpu, reset_count;
 	u64 last_reset_nsecs, total_nsecs;
 	u64 ready_nsecs, running_nsecs, paused_nsecs, halted_nsecs;
-	struct vmm_vcpu *vcpu = vmm_manager_vcpu(id);
+	struct vmm_vcpu *vcpu;
 
+	if (!argc) {
+		vmm_cprintf(cdev, "Must provide vcpu ID\n");
+		return VMM_EINVALID;
+	}
+	id = atoi(argv[0]);
+
+	vcpu = vmm_manager_vcpu(id);
 	if (!vcpu) {
 		vmm_cprintf(cdev, "Failed to find vcpu\n");
 		return VMM_EFAIL;
@@ -349,10 +554,13 @@ static int cmd_vcpu_dumpstat(struct vmm_chardev *cdev, int id)
 
 static const struct {
 	char *name;
-	int (*function) (struct vmm_chardev *, int);
+	int (*function) (struct vmm_chardev *, int, char **);
 } const command[] = {
 	{"help", cmd_vcpu_help},
 	{"list", cmd_vcpu_list},
+	{"orphan_list", cmd_vcpu_orphan_list},
+	{"normal_list", cmd_vcpu_normal_list},
+	{"monitor", cmd_vcpu_monitor},
 	{"reset", cmd_vcpu_reset},
 	{"kick", cmd_vcpu_kick},
 	{"pause", cmd_vcpu_pause},
@@ -363,9 +571,9 @@ static const struct {
 	{NULL, NULL},
 };
 	
-static int cmd_vcpu_exec(struct vmm_chardev *cdev, int argc, char **argv)
+static int cmd_vcpu_exec(struct vmm_chardev *cdev,
+			 int argc, char **argv)
 {
-	int id = -1;
 	int index = 0;
 
 	if ((argc == 1) || (argc > 3)) {
@@ -373,13 +581,10 @@ static int cmd_vcpu_exec(struct vmm_chardev *cdev, int argc, char **argv)
 		return VMM_EFAIL;
 	}
 
-	if (argc == 3) {
-		id = atoi(argv[2]);
-	}
-	
 	while (command[index].name) {
 		if (strcmp(argv[1], command[index].name) == 0) {
-			return command[index].function(cdev, id);
+			return command[index].function(cdev,
+						argc-2, &argv[2]);
 		}
 		index++;
 	}
