@@ -87,6 +87,15 @@ struct vmm_request_queue {
 	void *priv;
 };
 
+#define INIT_REQUEST_QUEUE(rq) \
+		do { \
+			INIT_SPIN_LOCK(&(rq)->lock); \
+			(rq)->make_request = NULL; \
+			(rq)->abort_request = NULL; \
+			(rq)->flush_cache = NULL; \
+			(rq)->priv = NULL; \
+		} while (0)
+
 /* Block device flags */
 #define VMM_BLOCKDEV_RDONLY				0x00000001
 #define VMM_BLOCKDEV_RW					0x00000002
@@ -102,7 +111,7 @@ struct vmm_blockdev {
 
 	char name[VMM_FIELD_NAME_SIZE];
 	char desc[VMM_FIELD_DESC_SIZE];
-	struct vmm_device *dev;
+	struct vmm_device dev;
 
 	u32 flags;
 	u64 start_lba;
@@ -201,8 +210,9 @@ int vmm_blockdev_unregister(struct vmm_blockdev *bdev);
 /** Find a block device in device driver framework */
 struct vmm_blockdev *vmm_blockdev_find(const char *name);
 
-/** Get block device with given number */
-struct vmm_blockdev *vmm_blockdev_get(int num);
+/** Iterate over each block device */
+int vmm_blockdev_iterate(struct vmm_blockdev *start, void *data,
+			 int (*fn)(struct vmm_blockdev *dev, void *data));
 
 /** Count number of block devices */
 u32 vmm_blockdev_count(void);
