@@ -25,9 +25,6 @@
 
 #include <vmm_types.h>
 #include <vmm_compiler.h>
-#include <vmm_spinlocks.h>
-#include <vmm_chardev.h>
-#include <vmm_heap.h>
 #include <libs/stacktrace.h>
 
 #define BUG_ON(x)							\
@@ -84,6 +81,8 @@ struct vmm_history {
 		(h)->tail = 0;						\
 	}								
 
+struct vmm_chardev;
+
 /** Check if a character is a control character */
 bool vmm_iscontrol(char c);
 
@@ -115,9 +114,37 @@ int vmm_snprintf(char *out, u32 out_sz, const char *format, ...);
 int vmm_cprintf(struct vmm_chardev *cdev, const char *format, ...);
 
 /** Print formatted string to default device */
-#define vmm_printf(...)	vmm_cprintf(NULL, __VA_ARGS__);
+#define vmm_printf(...)	vmm_cprintf(NULL, __VA_ARGS__)
 
-/** Panic & Print formatted message 
+/** Predefined log levels */
+enum vmm_print_level {
+	VMM_LOGLEVEL_EMERGENCY=0,
+	VMM_LOGLEVEL_ALERT=1,
+	VMM_LOGLEVEL_CRITICAL=2,
+	VMM_LOGLEVEL_ERROR=3,
+	VMM_LOGLEVEL_WARNING=4,
+	VMM_LOGLEVEL_NOTICE=5,
+	VMM_LOGLEVEL_INFO=6,
+};
+
+/** Print formatted string to default device if current
+ *  stdio log level is greater than or equal to specified level
+ */
+#define vmm_lprintf(level, msg...) \
+	do {								\
+		if (vmm_stdio_loglevel() >= (level))			\
+			vmm_printf(msg);				\
+	} while (0)
+
+#define vmm_lemergency(msg...)	vmm_lprintf(VMM_LOGLEVEL_EMERGENCY, msg)
+#define vmm_lalert(msg...)	vmm_lprintf(VMM_LOGLEVEL_ALERT, msg)
+#define vmm_lcritical(msg...)	vmm_lprintf(VMM_LOGLEVEL_CRITICAL, msg)
+#define vmm_lerror(msg...)	vmm_lprintf(VMM_LOGLEVEL_ERROR, msg)
+#define vmm_lwarning(msg...)	vmm_lprintf(VMM_LOGLEVEL_WARNING, msg)
+#define vmm_lnotice(msg...)	vmm_lprintf(VMM_LOGLEVEL_NOTICE, msg)
+#define vmm_linfo(msg...)	vmm_lprintf(VMM_LOGLEVEL_INFO, msg)
+
+/** Panic & Print formatted message
  * Note: This function is less verbose so perfer vmm_panic().
  */
 void __noreturn __vmm_panic(const char *format, ...);
@@ -133,20 +160,20 @@ void __noreturn __vmm_panic(const char *format, ...);
 int vmm_scanchars(struct vmm_chardev *cdev, char *ch, u32 num_ch, bool block);
 
 /** Get character from character device */
-char vmm_cgetc(struct vmm_chardev *cdev) ;
+char vmm_cgetc(struct vmm_chardev *cdev, bool lecho) ;
 
 /** Get character from default device */
-char vmm_getc(void);
+char vmm_getc(bool lecho);
 
 /** Get string from character device
  *  If history is NULL does not support UP/DN keys */
-char *vmm_cgets(struct vmm_chardev *cdev, char *s, int maxwidth, 
-		char endchar, struct vmm_history *history);
+char *vmm_cgets(struct vmm_chardev *cdev, char *s, int maxwidth,
+		char endchar, struct vmm_history *history, bool lecho);
 
 /** Get string from default device
  *  If history is NULL does not support UP/DN keys */
-char *vmm_gets(char *s, int maxwidth, char endchar, 
-		struct vmm_history *history);
+char *vmm_gets(char *s, int maxwidth, char endchar,
+	       struct vmm_history *history, bool lecho);
 
 /** Get default character device used by stdio */
 struct vmm_chardev *vmm_stdio_device(void);
@@ -154,6 +181,12 @@ struct vmm_chardev *vmm_stdio_device(void);
 /** Change default character device used by stdio */
 int vmm_stdio_change_device(struct vmm_chardev *cdev);
 
+/** Get log level used by stdio */
+long vmm_stdio_loglevel(void);
+
+/** Change log level used by stdio */
+void vmm_stdio_change_loglevel(long loglevel);
+
 /** Initialize standerd IO library */
 int vmm_stdio_init(void);
 
