@@ -98,7 +98,7 @@ static int amba_kmi_open(struct serio *io)
 	unsigned int divisor;
 	int ret;
 
-	ret = clk_enable(kmi->clk);
+	ret = clk_prepare_enable(kmi->clk);
 	if (ret)
 		goto out;
 
@@ -118,7 +118,7 @@ static int amba_kmi_open(struct serio *io)
 	return 0;
 
  clk_disable:
-	clk_disable(kmi->clk);
+	clk_disable_unprepare(kmi->clk);
  out:
 	return ret;
 }
@@ -130,7 +130,7 @@ static void amba_kmi_close(struct serio *io)
 	writeb(0, KMICR);
 
 	free_irq(kmi->irq, kmi);
-	clk_disable(kmi->clk);
+	clk_disable_unprepare(kmi->clk);
 }
 
 static int amba_kmi_driver_probe(struct vmm_device *dev,
@@ -151,13 +151,22 @@ static int amba_kmi_driver_probe(struct vmm_device *dev,
 	io->write	= amba_kmi_write;
 	io->open	= amba_kmi_open;
 	io->close	= amba_kmi_close;
-	strncpy(io->name, dev->node->name, sizeof(io->name));
-	strncpy(io->phys, dev->node->name, sizeof(io->phys));
+	if (strlcpy(io->name, dev->name, sizeof(io->name)) >=
+	    sizeof(io->name)) {
+		ret = -EOVERFLOW;
+		goto out;
+	}
+	if (strlcpy(io->phys, dev->name, sizeof(io->phys)) >=
+	    sizeof(io->phys)) {
+		ret = -EOVERFLOW;
+		goto out;
+	}
 	io->port_data	= kmi;
-	io->dev		= dev;
+	io->dev.parent	= dev;
 
 	kmi->io		= io;
-	ret = vmm_devtree_regmap(dev->node, (virtual_addr_t *)&kmi->base, 0);
+	ret = vmm_devtree_request_regmap(dev->node,
+				(virtual_addr_t *)&kmi->base, 0, "AMBA KMI");
 	if (ret) {
 		ret = -ENOMEM;
 		goto out;
@@ -165,7 +174,7 @@ static int amba_kmi_driver_probe(struct vmm_device *dev,
 
 	kmi->clk = clk_get(dev, "KMIREFCLK");
 	if (IS_ERR(kmi->clk)) {
-		ret = -EFAIL;
+		ret = PTR_ERR(kmi->clk);
 		goto unmap;
 	}
 
@@ -175,35 +184,37 @@ static int amba_kmi_driver_probe(struct vmm_device *dev,
 		goto unmap;
 	}
 
-	dev->priv = kmi;
+	vmm_devdrv_set_data(dev, kmi);
 
 	serio_register_port(kmi->io);
 
 	return VMM_OK;
 
  unmap:
-	vmm_devtree_regunmap(dev->node, (virtual_addr_t)kmi->base, 0);
+	vmm_devtree_regunmap_release(dev->node, (virtual_addr_t)kmi->base, 0);
  out:
-	kfree(kmi);
-	kfree(io);
+	if (kmi) kfree(kmi);
+	if (io) kfree(io);
 	return ret;
 }
 
 static int amba_kmi_driver_remove(struct vmm_device *dev)
 {
-	struct amba_kmi_port *kmi = (struct amba_kmi_port *)dev->priv;
+	struct amba_kmi_port *kmi =
+			(struct amba_kmi_port *)vmm_devdrv_get_data(dev);
 
-	dev->priv = NULL;
+	vmm_devdrv_set_data(dev, NULL);
 
 	serio_unregister_port(kmi->io);
-	vmm_devtree_regunmap(dev->node, (virtual_addr_t)kmi->base, 0);
+	clk_put(kmi->clk);
+	vmm_devtree_regunmap_release(dev->node, (virtual_addr_t)kmi->base, 0);
 	kfree(kmi);
 
 	return VMM_OK;
 }
 
 static struct vmm_devtree_nodeid amba_kmi_devid_table[] = {
-	{.type = "serio",.compatible = "pl050"},
+	{.type = "serio",.compatible = "arm,pl050"},
 	{.type = "serio",.compatible = "ambakmi"},
 	{ /* end of list */ },
 };
