@@ -47,7 +47,7 @@
 #define MODULE_DESC			"VESA 2.0 Framebuffer Driver"
 #define MODULE_AUTHOR			"Anup Patel"
 #define MODULE_LICENSE			"GPL"
-#define MODULE_IPRIORITY		(VMM_FB_CLASS_IPRIORITY+1)
+#define MODULE_IPRIORITY		(FB_CLASS_IPRIORITY+1)
 #define	MODULE_INIT			vesafb_init
 #define	MODULE_EXIT			vesafb_exit
 
@@ -207,8 +207,7 @@ static void vesafb_destroy(struct fb_info *info)
 {
 	fb_dealloc_cmap(&info->cmap);
 	if (info->screen_base)
-		vmm_host_memunmap((virtual_addr_t)info->screen_base, 
-				  info->screen_size);
+		vmm_host_memunmap((virtual_addr_t)info->screen_base);
 	release_mem_region(info->apertures->ranges[0].base, info->apertures->ranges[0].size);
 	framebuffer_release(info);
 }
@@ -261,7 +260,7 @@ static int vesafb_probe(struct vmm_device *dev,
 {
 	struct fb_info *info;
 	int i, err;
-	char *options = NULL;
+	const char *options = NULL;
 	unsigned int size_vmode;
 	unsigned int size_remap;
 	unsigned int size_total;
@@ -269,11 +268,11 @@ static int vesafb_probe(struct vmm_device *dev,
 	if (screen_info.orig_video_isVGA != VIDEO_TYPE_VLFB)
 		return -ENODEV;
 
-	options = vmm_devtree_attrval(dev->node, "options");
+	vmm_devtree_read_string(dev->node, "options", &options);
 	if (!options) {
 		return -ENODEV;
 	}
-	err = vesafb_setup(options);
+	err = vesafb_setup((char *)options);
 	if (err) {
 		return err;
 	}
@@ -537,12 +536,11 @@ static int vesafb_probe(struct vmm_device *dev,
 		goto err;
 	}
 	printk(KERN_INFO "vesafb: registered %s frame buffer device\n",
-	       info->dev->node->name);
+	       dev_name(&info->dev));
 	return 0;
 err:
 	if (info->screen_base)
-		vmm_host_memunmap((virtual_addr_t)info->screen_base, 
-				  info->screen_size);
+		vmm_host_memunmap((virtual_addr_t)info->screen_base);
 	framebuffer_release(info);
 	release_mem_region(vesafb_fix.smem_start, size_total);
 	return err;
@@ -558,8 +556,7 @@ static int vesafb_remove(struct vmm_device *dev)
 	if (info->cmap.len)
 		fb_dealloc_cmap(&info->cmap);
 	if (info->screen_base)
-		vmm_host_memunmap((virtual_addr_t)info->screen_base, 
-				  info->screen_size);
+		vmm_host_memunmap((virtual_addr_t)info->screen_base);
 	framebuffer_release(info);
 	release_mem_region(vesafb_fix.smem_start, vesafb_fix.smem_len);
 
