@@ -18,10 +18,10 @@
  *
  * @file arm_sysregs.c
  * @author Anup Patel (anup@brainfault.org)
- * @brief ARM Realview / Versatile Express System Registers emulator.
- * @details This source file implements the ARM system-registers emulator.
+ * @brief ARM Realview/Versatile/VExpress System Registers emulator.
+ * @details This source file emulates ARM system-registers.
  *
- * The source has been largely adapted from QEMU 0.14.xx hw/arm_sysctl.c 
+ * The source has been largely adapted from QEMU 1.0.xx hw/misc/arm_sysctl.c
  *
  * Status and system control registers for ARM RealView/Versatile boards.
  *
@@ -33,25 +33,23 @@
 
 #include <vmm_error.h>
 #include <vmm_heap.h>
+#include <vmm_stdio.h>
 #include <vmm_modules.h>
 #include <vmm_spinlocks.h>
 #include <vmm_devtree.h>
 #include <vmm_timer.h>
-#include <vmm_workqueue.h>
 #include <vmm_manager.h>
-#include <vmm_host_io.h>
 #include <vmm_devemu.h>
-#include <libs/stringlib.h>
 #include <libs/mathlib.h>
 
-#define MODULE_DESC			"Realview Sysctl Emulator"
+#define MODULE_DESC			"ARM Sysregs Emulator"
 #define MODULE_AUTHOR			"Anup Patel"
 #define MODULE_LICENSE			"GPL"
 #define MODULE_IPRIORITY		0
 #define	MODULE_INIT			arm_sysregs_emulator_init
 #define	MODULE_EXIT			arm_sysregs_emulator_exit
 
-#define LOCK_VAL			0x0000a05f
+#define LOCK_VALUE			0x0000a05f
 
 #define REALVIEW_SYSID_PBA8		0x01780500
 #define REALVIEW_PROCID_PBA8		0x0e000000
@@ -76,13 +74,11 @@
 
 struct arm_sysregs {
 	struct vmm_guest *guest;
-	vmm_spinlock_t lock;
+	vmm_rwlock_t lock;
 	u64 ref_100hz;
 	u64 ref_24mhz;
 	u32 mux_in_irq[2];
 	u32 mux_out_irq;
-	struct vmm_work reboot;
-	struct vmm_work shutdown;
 
 	u32 sys_id;
 	u32 leds;
@@ -98,6 +94,12 @@ struct arm_sysregs {
 	u32 sys_cfgctrl;
 	u32 sys_cfgstat;
 	u32 sys_clcd;
+	u32 mb_clock[6];
+	u32 *db_clock;
+	u32 db_num_vsensors;
+	u32 *db_voltage;
+	u32 db_num_clocks;
+	u32 *db_clock_reset;
 };
 
 static int board_id(struct arm_sysregs *s)
@@ -106,85 +108,82 @@ static int board_id(struct arm_sysregs *s)
     return (s->sys_id >> 16) & 0xfff;
 }
 
-static int arm_sysregs_emulator_read(struct vmm_emudev *edev,
-			    	     physical_addr_t offset, 
-				     void *dst, u32 dst_len)
+static int arm_sysregs_reg_read(struct arm_sysregs *s,
+				u32 offset, u32 *dst)
 {
 	int rc = VMM_OK;
-	u32 regval = 0x0;
 	u64 tdiff;
-	struct arm_sysregs *s = edev->priv;
 
-	vmm_spin_lock(&s->lock);
+	vmm_read_lock(&s->lock);
 
 	switch (offset & ~0x3) {
 	case 0x00: /* ID */
-		regval = s->sys_id;
+		*dst = s->sys_id;
 		break;
 	case 0x04: /* SW */
 		/* General purpose hardware switches.
 		We don't have a useful way of exposing these to the user.  */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0x08: /* LED */
-		regval = s->leds;
+		*dst = s->leds;
 		break;
 	case 0x20: /* LOCK */
-		regval = s->lockval;
+		*dst = s->lockval;
 		break;
 	case 0x0c: /* OSC0 */
 	case 0x10: /* OSC1 */
 	case 0x14: /* OSC2 */
 	case 0x18: /* OSC3 */
 	case 0x1c: /* OSC4 */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0x24: /* 100HZ */
 		tdiff = vmm_timer_timestamp() - s->ref_100hz;
-		regval = udiv64(tdiff, 10000000);
+		*dst = udiv64(tdiff, 10000000);
 		break;
 	case 0x28: /* CFGDATA1 */
-		regval = s->cfgdata1;
+		*dst = s->cfgdata1;
 		break;
 	case 0x2c: /* CFGDATA2 */
-		regval = s->cfgdata2;
+		*dst = s->cfgdata2;
 		break;
 	case 0x30: /* FLAGS */
-		regval = s->flags;
+		*dst = s->flags;
 		break;
 	case 0x38: /* NVFLAGS */
-		regval = s->nvflags;
+		*dst = s->nvflags;
 		break;
 	case 0x40: /* RESETCTL */
 		if (board_id(s) == BOARD_ID_VEXPRESS) {
 			/* reserved: RAZ/WI */
-			regval = 0;
+			*dst = 0;
 		} else {
-			regval = s->resetlevel;
+			*dst = s->resetlevel;
 		}
 		break;
 	case 0x44: /* PCICTL */
-		regval = 1;
+		*dst = 1;
 		break;
 	case 0x48: /* MCI */
-		regval = s->sys_mci;
+		*dst = s->sys_mci;
 		break;
 	case 0x4c: /* FLASH */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0x50: /* CLCD */
-		regval = s->sys_clcd;
+		*dst = s->sys_clcd;
 		break;
 	case 0x54: /* CLCDSER */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0x58: /* BOOTCS */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0x5c: /* 24MHz */
-		tdiff = vmm_timer_timestamp() - s->ref_100hz;
+		tdiff = vmm_timer_timestamp() - s->ref_24mhz;
 		/* Note: What we want is the below value 
-		 * regval = udiv64(tdiff * 24, 1000);
+		 * *dst = udiv64(tdiff * 24, 1000);
 		 * In integer arithmetic division by constant can be simplified
 		 * (a * 24) / 1000
 		 * = a * (24 / 1000)
@@ -195,16 +194,16 @@ static int arm_sysregs_emulator_read(struct vmm_emudev *edev,
 		 * ~ (a * 3) >> 7 + (a * 9) >> 14 
 		 */
 		tdiff = ((tdiff * 3) >> 7) + ((tdiff * 9) >> 14);
-		regval = tdiff;
+		*dst = tdiff;
 		break;
 	case 0x60: /* MISC */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0x84: /* PROCID0 */
-		regval = s->proc_id;
+		*dst = s->proc_id;
 		break;
 	case 0x88: /* PROCID1 */
-		regval = 0xff000000;
+		*dst = 0xff000000;
 		break;
 	case 0x64: /* DMAPSR0 */
 	case 0x68: /* DMAPSR1 */
@@ -222,107 +221,206 @@ static int arm_sysregs_emulator_read(struct vmm_emudev *edev,
 	case 0xc8: /* SYS_TEST_OSC2 */
 	case 0xcc: /* SYS_TEST_OSC3 */
 	case 0xd0: /* SYS_TEST_OSC4 */
-		regval = 0;
+		*dst = 0;
 		break;
 	case 0xa0: /* SYS_CFGDATA */
 		if (board_id(s) != BOARD_ID_VEXPRESS) {
 			rc = VMM_EFAIL;
 		} else {
-			regval = s->sys_cfgdata;
+			*dst = s->sys_cfgdata;
 		}
 		break;
 	case 0xa4: /* SYS_CFGCTRL */
 		if (board_id(s) != BOARD_ID_VEXPRESS) {
 			rc = VMM_EFAIL;
 		} else {
-			regval = s->sys_cfgctrl;
+			*dst = s->sys_cfgctrl;
 		}
 		break;
 	case 0xa8: /* SYS_CFGSTAT */
 		if (board_id(s) != BOARD_ID_VEXPRESS) {
 			rc = VMM_EFAIL;
 		} else {
-			regval = s->sys_cfgstat;
+			*dst = s->sys_cfgstat;
 		}
 		break;
 	case 0xd8: /* PLDCTL1 */
-		regval = 0;
+		*dst = 0;
 		break;
 	default:
 		rc = VMM_EFAIL;
 		break;
 	}
 
-	vmm_spin_unlock(&s->lock);
-
-	if (!rc) {
-		regval = (regval >> ((offset & 0x3) * 8));
-		switch (dst_len) {
-		case 1:
-			*(u8 *)dst = regval & 0xFF;
-			break;
-		case 2:
-			*(u16 *)dst = vmm_cpu_to_le16(regval & 0xFFFF);
-			break;
-		case 4:
-			*(u32 *)dst = vmm_cpu_to_le32(regval);
-			break;
-		default:
-			rc = VMM_EFAIL;
-			break;
-		};
-	}
+	vmm_read_unlock(&s->lock);
 
 	return rc;
 }
 
-static void arm_sysregs_shutdown(struct vmm_work *w)
+/* SYS_CFGCTRL functions */
+#define SYS_CFG_OSC 1
+#define SYS_CFG_VOLT 2
+#define SYS_CFG_AMP 3
+#define SYS_CFG_TEMP 4
+#define SYS_CFG_RESET 5
+#define SYS_CFG_SCC 6
+#define SYS_CFG_MUXFPGA 7
+#define SYS_CFG_SHUTDOWN 8
+#define SYS_CFG_REBOOT 9
+#define SYS_CFG_DVIMODE 11
+#define SYS_CFG_POWER 12
+#define SYS_CFG_ENERGY 13
+
+/* SYS_CFGCTRL site field values */
+#define SYS_CFG_SITE_MB 0
+#define SYS_CFG_SITE_DB1 1
+#define SYS_CFG_SITE_DB2 2
+
+/**
+ * Handle a VExpress SYS_CFGCTRL register read. On success, return true and
+ * write the read value to *val. On failure, return false (and val may
+ * or may not be written to).
+ *
+ * vexpress_cfgctrl_read:
+ * @s: arm_sysctl_state pointer
+ * @dcc, @function, @site, @position, @device: split out values from
+ * SYS_CFGCTRL register
+ * @val: pointer to where to put the read data on success
+ */
+static bool vexpress_cfgctrl_read(struct arm_sysregs *s, u32 dcc,
+                                  u32 function, u32 site,
+                                  u32 position, u32 device,
+                                  u32 *val)
 {
-	struct arm_sysregs *s = container_of(w, struct arm_sysregs, shutdown);
+	/* We don't support anything other than DCC 0, board stack position 0
+	 * or sites other than motherboard/daughterboard:
+	 */
+	if (dcc != 0 || position != 0 ||
+	   (site != SYS_CFG_SITE_MB && site != SYS_CFG_SITE_DB1)) {
+		goto cfgctrl_unimp;
+	}
 
-	vmm_manager_guest_reset(s->guest);
-}
-
-static void arm_sysregs_reboot(struct vmm_work *w)
-{
-	struct arm_sysregs *s = container_of(w, struct arm_sysregs, reboot);
-
-	vmm_manager_guest_reset(s->guest);
-	vmm_manager_guest_kick(s->guest);
-}
-
-static int arm_sysregs_emulator_write(struct vmm_emudev *edev,
-				      physical_addr_t offset, 
-				      void *src, u32 src_len)
-{
-	int rc = VMM_OK, i;
-	u32 regmask = 0x0, regval = 0x0;
-	struct arm_sysregs *s = edev->priv;
-
-	switch (src_len) {
-	case 1:
-		regmask = 0xFFFFFF00;
-		regval = *(u8 *)src;
+	switch (function) {
+	case SYS_CFG_VOLT:
+		if (site == SYS_CFG_SITE_DB1 && device < s->db_num_vsensors) {
+			*val = s->db_voltage[device];
+			return TRUE;
+		}
+		if (site == SYS_CFG_SITE_MB && device == 0) {
+			/* There is only one motherboard voltage sensor:
+			 * VIO : 3.3V : bus voltage between mother and
+			 * daughterboard
+			 */
+			*val = 3300000;
+			return TRUE;
+		}
 		break;
-	case 2:
-		regmask = 0xFFFF0000;
-		regval = vmm_le16_to_cpu(*(u16 *)src);
-		break;
-	case 4:
-		regmask = 0x00000000;
-		regval = vmm_le32_to_cpu(*(u32 *)src);
+	case SYS_CFG_OSC:
+		if (site == SYS_CFG_SITE_MB && device < sizeof(s->mb_clock)) {
+			/* motherboard clock */
+			*val = s->mb_clock[device];
+			return TRUE;
+		}
+		if (site == SYS_CFG_SITE_DB1 && device < s->db_num_clocks) {
+			/* daughterboard clock */
+			*val = s->db_clock[device];
+			return TRUE;
+		}
 		break;
 	default:
-		return VMM_EFAIL;
 		break;
 	};
 
-	for (i = 0; i < (offset & 0x3); i++) {
-		regmask = (regmask << 8) | ((regmask >> 24) & 0xFF);
-	}
-	regval = (regval << ((offset & 0x3) * 8));
+cfgctrl_unimp:
+	vmm_printf("%s: Unimplemented SYS_CFGCTRL read of function "
+		   "0x%x DCC 0x%x site 0x%x position 0x%x device 0x%x\n",
+		   __func__, function, dcc, site, position, device);
+	return FALSE;
+}
 
-	vmm_spin_lock(&s->lock);
+/**
+ * Handle a VExpress SYS_CFGCTRL register write. On success, return true.
+ * On failure, return false.
+ *
+ * @s: arm_sysctl_state pointer
+ * @dcc, @function, @site, @position, @device: split out values from
+ * SYS_CFGCTRL register
+ * @val: data to write
+ */
+static bool vexpress_cfgctrl_write(struct arm_sysregs *s, u32 dcc,
+                                   u32 function, u32 site,
+                                   u32 position, u32 device,
+                                   u32 val)
+{
+	/* We don't support anything other than DCC 0, board stack position 0
+	 * or sites other than motherboard/daughterboard:
+	 */
+	if (dcc != 0 || position != 0 ||
+	    (site != SYS_CFG_SITE_MB && site != SYS_CFG_SITE_DB1)) {
+		goto cfgctrl_unimp;
+	}
+
+	switch (function) {
+	case SYS_CFG_OSC:
+		if (site == SYS_CFG_SITE_MB && device < sizeof(s->mb_clock)) {
+			/* motherboard clock */
+			s->mb_clock[device] = val;
+			return TRUE;
+		}
+		if (site == SYS_CFG_SITE_DB1 && device < s->db_num_clocks) {
+			/* daughterboard clock */
+			s->db_clock[device] = val;
+			return TRUE;
+		}
+		break;
+	case SYS_CFG_MUXFPGA:
+		if (site == SYS_CFG_SITE_MB && device == 0) {
+			/* Select whether video output comes from motherboard
+			 * or daughterboard: ignore it as Xvisor doesn't
+			 * support this.
+			 */
+			return TRUE;
+		}
+		break;
+	case SYS_CFG_SHUTDOWN:
+		if (site == SYS_CFG_SITE_MB && device == 0) {
+			vmm_manager_guest_shutdown_request(s->guest);
+			return TRUE;
+		}
+		break;
+	case SYS_CFG_REBOOT:
+		if (site == SYS_CFG_SITE_MB && device == 0) {
+			vmm_manager_guest_reboot_request(s->guest);
+			return TRUE;
+		}
+		break;
+	case SYS_CFG_DVIMODE:
+		if (site == SYS_CFG_SITE_MB && device == 0) {
+			/* Selecting DVI mode is meaningless for Xvisor:
+			 * we will always display the output correctly
+			 * according to the pixel height/width programmed
+			 * into the CLCD controller.
+			 */
+			return TRUE;
+		}
+		break;
+	default:
+		break;
+	};
+
+cfgctrl_unimp:
+	vmm_printf("%s: Unimplemented SYS_CFGCTRL write of function "
+		   "0x%x DCC 0x%x site 0x%x position 0x%x device 0x%x\n",
+		   __func__, function, dcc, site, position, device);
+	return FALSE;
+}
+
+static int arm_sysregs_reg_write(struct arm_sysregs *s,
+				 u32 offset, u32 regmask, u32 regval)
+{
+	int rc = VMM_OK;
+
+	vmm_write_lock(&s->lock);
 
 	switch (offset & ~0x3) {
 	case 0x08: /* LED */
@@ -338,7 +436,7 @@ static int arm_sysregs_emulator_write(struct vmm_emudev *edev,
 		break;
 	case 0x20: /* LOCK */
 		s->lockval &= regmask;
-		if (regval == LOCK_VAL) {
+		if (regval == LOCK_VALUE) {
 			s->lockval = regval;
 		} else {
 			s->lockval = regval & 0x7fff;
@@ -369,33 +467,33 @@ static int arm_sysregs_emulator_write(struct vmm_emudev *edev,
 	case 0x40: /* RESETCTL */
 		switch (board_id(s)) {
 		case BOARD_ID_PB926:
-			if (s->lockval == LOCK_VAL) {
+			if (s->lockval == LOCK_VALUE) {
 				s->resetlevel &= regmask;
 				s->resetlevel |= regval;
 				if (s->resetlevel & 0x100) {
-					vmm_workqueue_schedule_work(NULL, 
-								&s->reboot);
+					vmm_manager_guest_reboot_request(
+								s->guest);
 				}
 			}
 			break;
 		case BOARD_ID_PBX:
 		case BOARD_ID_PBA8:
-			if (s->lockval == LOCK_VAL) {
+			if (s->lockval == LOCK_VALUE) {
 				s->resetlevel &= regmask;
 				s->resetlevel |= regval;
 				if (s->resetlevel & 0x04) {
-					vmm_workqueue_schedule_work(NULL, 
-								&s->reboot);
+					vmm_manager_guest_reboot_request(
+								s->guest);
 				}
 			}
 			break;
 		case BOARD_ID_EB:
-			if (s->lockval == LOCK_VAL) {
+			if (s->lockval == LOCK_VALUE) {
 				s->resetlevel &= regmask;
 				s->resetlevel |= regval;
 				if (s->resetlevel & 0x08) {
-					vmm_workqueue_schedule_work(NULL, 
-								&s->reboot);
+					vmm_manager_guest_reboot_request(
+								s->guest);
 				}
 			}
 			break;
@@ -472,19 +570,40 @@ static int arm_sysregs_emulator_write(struct vmm_emudev *edev,
 			rc =  VMM_EFAIL;
 			break;
 		}
+		/* Undefined bits [19:18] are RAZ/WI, and writing to
+		 * the start bit just triggers the action; it always
+		 * reads as zero.
+		 */
 		s->sys_cfgctrl &= regmask;
 		s->sys_cfgctrl |= regval & ~(3 << 18);
-		s->sys_cfgstat = 1;            /* complete */
-		switch (s->sys_cfgctrl) {
-		case 0xc0800000: /* SYS_CFG_SHUTDOWN to motherboard */
-			vmm_workqueue_schedule_work(NULL, &s->shutdown);
-			break;
-		case 0xc0900000: /* SYS_CFG_REBOOT to motherboard */
-			vmm_workqueue_schedule_work(NULL, &s->reboot);
-			break;
-		default:
-			s->sys_cfgstat |= 2;        /* error */
+		if (s->sys_cfgctrl & (1 << 31)) {
+			/* Start bit set -- actually do something */
+			u32 dcc = (s->sys_cfgctrl >> 26) & 0xF;
+			u32 function = (s->sys_cfgctrl >> 20) & 0x3F;
+			u32 site = (s->sys_cfgctrl >> 16) & 0x3;
+			u32 position = (s->sys_cfgctrl >> 12) & 0xF;
+			u32 device = (s->sys_cfgctrl >> 0) & 0xFFF;
+			s->sys_cfgstat = 1;			/* complete */
+			if (s->sys_cfgctrl & (1 << 30)) {
+				if (!vexpress_cfgctrl_write(s, dcc,
+							    function, site,
+							    position, device,
+							    s->sys_cfgdata)) {
+					s->sys_cfgstat |= 2;	/* error */
+				}
+			} else {
+				u32 val;
+				if (!vexpress_cfgctrl_read(s, dcc,
+							   function, site,
+							   position, device,
+							   &val)) {
+					s->sys_cfgstat |= 2;	/* error */
+				} else {
+					s->sys_cfgdata = val;
+				}
+			}
 		}
+		s->sys_cfgctrl &= ~(1 << 31);
 		break;
 	case 0xa8: /* SYS_CFGSTAT */
 		if (board_id(s) != BOARD_ID_VEXPRESS) {
@@ -501,26 +620,96 @@ static int arm_sysregs_emulator_write(struct vmm_emudev *edev,
 		break;
 	}
 
-	vmm_spin_unlock(&s->lock);
+	vmm_write_unlock(&s->lock);
 
 	return rc;
 }
 
+static int arm_sysregs_emulator_read8(struct vmm_emudev *edev,
+				      physical_addr_t offset, 
+				      u8 *dst)
+{
+	int rc;
+	u32 regval = 0x0;
+
+	rc = arm_sysregs_reg_read(edev->priv, offset, &regval);
+	if (!rc) {
+		*dst = regval & 0xFF;
+	}
+
+	return rc;
+}
+
+static int arm_sysregs_emulator_read16(struct vmm_emudev *edev,
+				       physical_addr_t offset, 
+				       u16 *dst)
+{
+	int rc;
+	u32 regval = 0x0;
+
+	rc = arm_sysregs_reg_read(edev->priv, offset, &regval);
+	if (!rc) {
+		*dst = regval & 0xFFFF;
+	}
+
+	return rc;
+}
+
+static int arm_sysregs_emulator_read32(struct vmm_emudev *edev,
+				       physical_addr_t offset, 
+				       u32 *dst)
+{
+	return arm_sysregs_reg_read(edev->priv, offset, dst);
+}
+
+static int arm_sysregs_emulator_write8(struct vmm_emudev *edev,
+				       physical_addr_t offset, 
+				       u8 src)
+{
+	return arm_sysregs_reg_write(edev->priv, offset, 0xFFFFFF00, src);
+}
+
+static int arm_sysregs_emulator_write16(struct vmm_emudev *edev,
+					physical_addr_t offset, 
+					u16 src)
+{
+	return arm_sysregs_reg_write(edev->priv, offset, 0xFFFF0000, src);
+}
+
+static int arm_sysregs_emulator_write32(struct vmm_emudev *edev,
+					physical_addr_t offset, 
+					u32 src)
+{
+	return arm_sysregs_reg_write(edev->priv, offset, 0x00000000, src);
+}
+
 static int arm_sysregs_emulator_reset(struct vmm_emudev *edev)
 {
+	int i;
 	struct arm_sysregs *s = edev->priv;
 
-	vmm_spin_lock(&s->lock);
+	vmm_write_lock(&s->lock);
 
 	s->ref_100hz = vmm_timer_timestamp();
 	s->ref_24mhz = s->ref_100hz;
 
 	s->leds = 0;
-	s->lockval = 0x10000;
+	s->lockval = 0;
 	s->cfgdata1 = 0;
 	s->cfgdata2 = 0;
 	s->flags = 0;
 	s->resetlevel = 0;
+	/* Motherboard oscillators (in Hz) */
+	s->mb_clock[0] = 50000000; /* Static memory clock: 50MHz */
+	s->mb_clock[1] = 23750000; /* motherboard CLCD clock: 23.75MHz */
+	s->mb_clock[2] = 24000000; /* IO FPGA peripheral clock: 24MHz */
+	s->mb_clock[3] = 24000000; /* IO FPGA reserved clock: 24MHz */
+	s->mb_clock[4] = 24000000; /* System bus global clock: 24MHz */
+	s->mb_clock[5] = 24000000; /* IO FPGA reserved clock: 24MHz */
+	/* Daughterboard oscillators: reset from property values */
+	for (i = 0; i < s->db_num_clocks; i++) {
+		s->db_clock[i] = s->db_clock_reset[i];
+	}
 	if (board_id(s) == BOARD_ID_VEXPRESS) {
 		/* On VExpress this register will RAZ/WI */
 		s->sys_clcd = 0;
@@ -529,7 +718,7 @@ static int arm_sysregs_emulator_reset(struct vmm_emudev *edev)
 		s->sys_clcd = 0x1f00;
 	}
 
-	vmm_spin_unlock(&s->lock);
+	vmm_write_unlock(&s->lock);
 
 	return VMM_OK;
 }
@@ -540,11 +729,12 @@ static void arm_sysregs_irq_handle(u32 irq, int cpu, int level, void *opaque)
 	int bit;
 	struct arm_sysregs *s = opaque;
 
+	vmm_write_lock(&s->lock);
+
 	if (s->mux_in_irq[0] == irq) {
 		/* For PB926 and EB write-protect is bit 2 of SYS_MCI;
 		 * for all later boards it is bit 1.
 		 */
-		vmm_spin_lock(&s->lock);
 		bit = 2;
 		if ((board_id(s) == BOARD_ID_PB926) || 
 		    (board_id(s) == BOARD_ID_EB)) {
@@ -554,61 +744,85 @@ static void arm_sysregs_irq_handle(u32 irq, int cpu, int level, void *opaque)
 		if (level) {
 			s->sys_mci |= bit;
 		}
-		vmm_spin_unlock(&s->lock);
 	} else if (s->mux_in_irq[1] == irq) {
-		vmm_spin_lock(&s->lock);
 		s->sys_mci &= ~1;
 		if (level) {
 			s->sys_mci |= 1;
 		}
-		vmm_spin_unlock(&s->lock);
 	}
+
+	vmm_write_unlock(&s->lock);
 }
 
 static int arm_sysregs_emulator_probe(struct vmm_guest *guest,
 				   struct vmm_emudev *edev,
 				   const struct vmm_devtree_nodeid *eid)
 {
-	int rc = VMM_OK;
-	const char * attr;
+	int i, j, rc = VMM_OK;
 	struct arm_sysregs *s;
 
 	s = vmm_zalloc(sizeof(struct arm_sysregs));
 	if (!s) {
-		rc = VMM_EFAIL;
+		rc = VMM_ENOMEM;
 		goto arm_sysregs_emulator_probe_done;
 	}
 
 	s->guest = guest;
-	INIT_SPIN_LOCK(&s->lock);
+	INIT_RW_LOCK(&s->lock);
 	s->ref_100hz = vmm_timer_timestamp();
 	s->ref_24mhz = s->ref_100hz;
 
 	if (eid->data) {
-		s->sys_id = ((u32 *)eid->data)[0];
-		s->proc_id = ((u32 *)eid->data)[1];
+		i = 0;
+		s->sys_id = ((u32 *)eid->data)[i++];
+		s->proc_id = ((u32 *)eid->data)[i++];
+		s->db_num_vsensors = ((u32 *)eid->data)[i++];
+		if (s->db_num_vsensors) {
+			s->db_voltage =
+				vmm_zalloc(sizeof(u32)*s->db_num_vsensors);
+			if (!s->db_voltage) {
+				rc = VMM_ENOMEM;
+				goto arm_sysregs_emulator_probe_freestate_fail;
+			}
+			for (j = 0; j < s->db_num_vsensors; j++) {
+				s->db_voltage[j] = ((u32 *)eid->data)[i++];
+			}
+		}
+		s->db_num_clocks = ((u32 *)eid->data)[i++];
+		if (s->db_num_clocks) {
+			s->db_clock =
+				vmm_zalloc(sizeof(u32)*s->db_num_clocks);
+			if (!s->db_clock) {
+				rc = VMM_ENOMEM;
+				goto arm_sysregs_emulator_probe_freevolt_fail;
+			}
+			s->db_clock_reset =
+				vmm_zalloc(sizeof(u32)*s->db_num_clocks);
+			if (!s->db_clock_reset) {
+				vmm_free(s->db_clock);
+				s->db_clock = NULL;
+				rc = VMM_ENOMEM;
+				goto arm_sysregs_emulator_probe_freevolt_fail;
+			}
+			for (j = 0; j < s->db_num_clocks; j++) {
+				s->db_clock_reset[j] = ((u32 *)eid->data)[i++];
+			}
+		}
 	}
 
-	attr = vmm_devtree_attrval(edev->node, "mux_in_irq");
-	if (attr) {
-		s->mux_in_irq[0] = ((u32 *)attr)[0];
-		s->mux_in_irq[1] = ((u32 *)attr)[1];
-	} else {
-		rc = VMM_EFAIL;
-		goto arm_sysregs_emulator_probe_freestate_fail;
+	rc = vmm_devtree_read_u32_array(edev->node, "mux_in_irq",
+					s->mux_in_irq,
+					array_size(s->mux_in_irq));
+	if (rc) {
+		goto arm_sysregs_emulator_probe_freeclock_fail;
 	}
 
-	attr = vmm_devtree_attrval(edev->node, "mux_out_irq");
-	if (attr) {
-		s->mux_out_irq = ((u32 *)attr)[0];
-	} else {
-		rc = VMM_EFAIL;
-		goto arm_sysregs_emulator_probe_freestate_fail;
+	rc = vmm_devtree_read_u32(edev->node, "mux_out_irq",
+				    &s->mux_out_irq);
+	if (rc) {
+		goto arm_sysregs_emulator_probe_freeclock_fail;
 	}
 
-	INIT_WORK(&s->shutdown, arm_sysregs_shutdown);
-	INIT_WORK(&s->reboot, arm_sysregs_reboot);
-
 	vmm_devemu_register_irq_handler(guest, s->mux_in_irq[0],
 					edev->node->name, 
 					arm_sysregs_irq_handle, s);
@@ -620,6 +834,17 @@ static int arm_sysregs_emulator_probe(struct vmm_guest *guest,
 
 	goto arm_sysregs_emulator_probe_done;
 
+arm_sysregs_emulator_probe_freeclock_fail:
+	if (s->db_clock) {
+		vmm_free(s->db_clock);
+	}
+	if (s->db_clock_reset) {
+		vmm_free(s->db_clock_reset);
+	}
+arm_sysregs_emulator_probe_freevolt_fail:
+	if (s->db_voltage) {
+		vmm_free(s->db_voltage);
+	}
 arm_sysregs_emulator_probe_freestate_fail:
 	vmm_free(s);
 arm_sysregs_emulator_probe_done:
@@ -630,46 +855,86 @@ static int arm_sysregs_emulator_remove(struct vmm_emudev *edev)
 {
 	struct arm_sysregs *s = edev->priv;
 
-	if (s) {
-		vmm_devemu_unregister_irq_handler(s->guest, s->mux_in_irq[0], 
-						  arm_sysregs_irq_handle, s);
-		vmm_devemu_unregister_irq_handler(s->guest, s->mux_in_irq[1], 
-						  arm_sysregs_irq_handle, s);
-		vmm_free(s);
-		edev->priv = NULL;
+	if (!s) {
+		return VMM_EFAIL;
 	}
 
+	vmm_devemu_unregister_irq_handler(s->guest, s->mux_in_irq[0], 
+					  arm_sysregs_irq_handle, s);
+	vmm_devemu_unregister_irq_handler(s->guest, s->mux_in_irq[1], 
+					  arm_sysregs_irq_handle, s);
+	if (s->db_clock) {
+		vmm_free(s->db_clock);
+	}
+	if (s->db_clock_reset) {
+		vmm_free(s->db_clock_reset);
+	}
+	if (s->db_voltage) {
+		vmm_free(s->db_voltage);
+	}
+	vmm_free(s);
+	edev->priv = NULL;
+
 	return VMM_OK;
 }
 
 static u32 versatilepb_sysids[] = {
 	/* === VersatilePB === */
-	/* sys_id */ VERSATILEPB_SYSID_ARM926, 
-	/* proc_id */ VERSATILEPB_PROCID_ARM926, 
+	/* sys_id */ VERSATILEPB_SYSID_ARM926,
+	/* proc_id */ VERSATILEPB_PROCID_ARM926,
+	/* len-db-voltage */ 0,
+	/* len-db-clock */ 0,
 };
 
 static u32 realview_ebmpcore_sysids[] = {
 	/* === Realview-EB-MPCore === */
-	/* sys_id */ REALVIEW_SYSID_EB11MP, 
-	/* proc_id */ REALVIEW_PROCID_EB11MP, 
+	/* sys_id */ REALVIEW_SYSID_EB11MP,
+	/* proc_id */ REALVIEW_PROCID_EB11MP,
+	/* len-db-voltage */ 0,
+	/* len-db-clock */ 0,
 };
 
 static u32 realview_pba8_sysids[] = {
 	/* === Realview-PB-aA8 === */
-	/* sys_id */ REALVIEW_SYSID_PBA8, 
-	/* proc_id */ REALVIEW_PROCID_PBA8, 
+	/* sys_id */ REALVIEW_SYSID_PBA8,
+	/* proc_id */ REALVIEW_PROCID_PBA8,
+	/* len-db-voltage */ 0,
+	/* len-db-clock */ 0,
 };
 
 static u32 vexpress_a9_sysids[] = {
 	/* === VExpress-A9 === */
-	/* sys_id */ VEXPRESS_SYSID_CA9, 
-	/* proc_id */ VEXPRESS_PROCID_CA9, 
+	/* sys_id */ VEXPRESS_SYSID_CA9,
+	/* proc_id */ VEXPRESS_PROCID_CA9,
+	/* len-db-voltage */ 6,
+	/* db-voltage */ 1000000, /* VD10 : 1.0V : SoC internal logic voltage */
+	/* db-voltage */ 1000000, /* VD10_S2 : 1.0V : PL310, L2 cache, RAM, non-PL310 logic */
+	/* db-voltage */ 1000000, /* VD10_S3 : 1.0V : Cortex-A9, cores, MPEs, SCU, PL310 logic */
+	/* db-voltage */ 1800000, /* VCC1V8 : 1.8V : DDR2 SDRAM, test chip DDR2 I/O supply */
+	/* db-voltage */ 900000, /* DDR2VTT : 0.9V : DDR2 SDRAM VTT termination voltage */
+	/* db-voltage */ 3300000, /* VCC3V3 : 3.3V : local board supply for misc external logic */
+	/* len-db-clock */ 3,
+	/* db-clock */ 45000000, /* AMBA AXI ACLK: 45MHz */
+	/* db-clock */ 23750000, /* daughterboard CLCD clock: 23.75MHz */
+	/* db-clock */ 66670000, /* Test chip reference clock: 66.67MHz */
 };
 
 static u32 vexpress_a15_sysids[] = {
 	/* === VExpress-A15 === */
-	/* sys_id */ VEXPRESS_SYSID_CA15, 
-	/* proc_id */ VEXPRESS_PROCID_CA15, 
+	/* sys_id */ VEXPRESS_SYSID_CA15,
+	/* proc_id */ VEXPRESS_PROCID_CA15,
+	/* len-db-voltage */ 1,
+	/* db-voltage */ 900000, /* Vcore: 0.9V : CPU core voltage */
+	/* len-db-clock */ 9,
+	/* db-clock */ 60000000, /* OSCCLK0: 60MHz : CPU_CLK reference */
+	/* db-clock */ 0, /* OSCCLK1: reserved */
+	/* db-clock */ 0, /* OSCCLK2: reserved */
+	/* db-clock */ 0, /* OSCCLK3: reserved */
+	/* db-clock */ 40000000, /* OSCCLK4: 40MHz : external AXI master clock */
+	/* db-clock */ 23750000, /* OSCCLK5: 23.75MHz : HDLCD PLL reference */
+	/* db-clock */ 50000000, /* OSCCLK6: 50MHz : static memory controller clock */
+	/* db-clock */ 60000000, /* OSCCLK7: 60MHz : SYSCLK reference */
+	/* db-clock */ 40000000, /* OSCCLK8: 40MHz : DDR2 PLL reference */
 };
 
 static struct vmm_devtree_nodeid arm_sysregs_emuid_table[] = {
@@ -699,9 +964,14 @@ static struct vmm_devtree_nodeid arm_sysregs_emuid_table[] = {
 static struct vmm_emulator arm_sysregs_emulator = {
 	.name = "arm_sysregs",
 	.match_table = arm_sysregs_emuid_table,
+	.endian = VMM_DEVEMU_LITTLE_ENDIAN,
 	.probe = arm_sysregs_emulator_probe,
-	.read = arm_sysregs_emulator_read,
-	.write = arm_sysregs_emulator_write,
+	.read8 = arm_sysregs_emulator_read8,
+	.write8 = arm_sysregs_emulator_write8,
+	.read16 = arm_sysregs_emulator_read16,
+	.write16 = arm_sysregs_emulator_write16,
+	.read32 = arm_sysregs_emulator_read32,
+	.write32 = arm_sysregs_emulator_write32,
 	.reset = arm_sysregs_emulator_reset,
 	.remove = arm_sysregs_emulator_remove,
 };
